// this file contains: main BView descendant for displaying Squeak
//
// authors: Trey Bordeau, Colin Sarsfield
//
// Please read the file named License for license information.
// 

#include "sqBeView.h"
#include "sqBeApp.h"
#include "sqBeIOPrims.h"
#include "sqBeGlobal.h"
#include "sqBeDirectWindow.h"
#include "sqBeSharedWindow.h"

#include <Bitmap.h>
#include <Locker.h>

bool CursorIsInView = false; // in the mouse cursor in the SqueakView
int ButtonState = 0; // mouse button and modifier state when mouse button went down or 0 if not pressed 
BLocker DeferedRegLock; // lock for synchronized access to DeferedRegion
BRegion DeferedRegion; // areas for defered update
bool GoAheadAndDraw = false; // draw the update region now instead of just adding to the deferred region
bool HasPeekKey = false; // Is their a peek value currently being held?
bool InDirectWindowDraw = false; // is window being drawn using BDirectWindow at this very moment?
port_id KeyBuf; // port used for keyboard buffer
int32 PeekKey = 0; // since you can't "peek" a message from a port this holds the peek value unit the next get
BPoint SavedMousePosition; // mouse position when window is inactive
BBitmap *Scr = 0; // bitmap mirror of the Screen form (only used for BWindow implementation)
bool UsingDirectWindow = false; // true if using a direct window class
bool WindowActive = false; // true if the Squeak window is the active window

void ConvertButtonState(uint32 buttons);

// SqueakView //

SqueakView::SqueakView(BRect rect) : BView(rect, "MainView", B_FOLLOW_ALL_SIDES, B_NAVIGABLE | B_WILL_DRAW) { 
	SetViewColor(B_TRANSPARENT_32_BIT);
}
	
void SqueakView::Draw(BRect updateRect) {
	DEBUG_OUT("Draw");
	
	if (UsingDirectWindow) {
		// this insures that when the window is resized to be bigger and Squeak has not changed its
		// window size the area between squeak's window rectangle and BeOS's window rectangle
		// will be painted white
		FillRect(updateRect, B_SOLID_LOW);
		((SqueakDirectWindow *)SqWindow)->Invalidate(updateRect.left, updateRect.top, updateRect.right, updateRect.bottom);
		GoAheadAndDraw = true;
	} else {
		// has the screen bitmap been created?				
		if (Scr != 0)
			{
			// if so, draw it
			// this seems to be aware of the update region
			// and do it faster (than painting individual rects), not surprising really, gotta love API
			DrawBitmapAsync(Scr);
		
			// fill in anywhere outside the squeak space
			BRegion rgnDraw;
			rgnDraw.Set(updateRect);
			rgnDraw.Exclude(Scr->Bounds());
			FillRegion(&rgnDraw, B_SOLID_LOW);
		} else
			// otherwise fill with white
			FillRect(Frame(), B_SOLID_LOW);
	}
	DEBUG_OUT("Draw/ret");
}

void SqueakView::MouseDown(BPoint where) {
	DEBUG_OUT("SqueakView::MouseDown");

	SaveMouseData(where);

	SetMouseEventMask(B_POINTER_EVENTS, B_LOCK_WINDOW_FOCUS);
   	
	BView::MouseDown(where);
	DEBUG_OUT("SqueakView::MouseDown/ret");
}

void SqueakView::MouseMoved(BPoint where, uint32 code, const BMessage *a_message) {	
	DEBUG_OUT("SqueakView::MouseMoved");

	switch (code) {
		case B_ENTERED_VIEW:
			CursorIsInView = true;
			be_app->SetCursor(&SqueakCursor);
			break;
		case B_EXITED_VIEW:
			CursorIsInView = false;
			be_app->SetCursor(B_HAND_CURSOR);
			break;
		case B_INSIDE_VIEW:
		case B_OUTSIDE_VIEW:
			SaveMouseData(where);
	}
	
	BView::MouseMoved(where, code, a_message);
	DEBUG_OUT("SqueakView::MouseMoved/ret");
}

void SqueakView::MouseUp(BPoint where) {
	DEBUG_OUT("SqueakView::MouseUp");
	SaveMouseData(where);
	
	BView::MouseUp(where);
	DEBUG_OUT("SqueakView::MouseUp/ret");
}

void SqueakView::SaveMouseData(const BPoint where) {
	SavedMousePosition = where;
	int32 Buttons = 0, Modifiers = 0;
	Window()->CurrentMessage()->FindInt32("buttons", &Buttons);
	Window()->CurrentMessage()->FindInt32("modifiers", &Modifiers);
	ConvertButtonState(Buttons);
	ConvertModifiers(Modifiers);
}


// support functions

void ConvertButtonState(uint32 buttons) { 
	DEBUG_OUT("ConvertButtonState");
	ButtonState &= ~7;
	// pass mouse button info into structure
	ButtonState |= ((buttons & B_PRIMARY_MOUSE_BUTTON) ? 4 : 0)
		| ((buttons & B_SECONDARY_MOUSE_BUTTON) ? 2 : 0)
		| ((buttons & B_TERTIARY_MOUSE_BUTTON) ? 1 : 0);
	DEBUG_OUT("ConvertButtonState/ret");
}
	
